# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from dropbox_sign.models.signature_request_response_attachment import (
    SignatureRequestResponseAttachment,
)
from dropbox_sign.models.signature_request_response_custom_field_base import (
    SignatureRequestResponseCustomFieldBase,
)
from dropbox_sign.models.signature_request_response_data_base import (
    SignatureRequestResponseDataBase,
)
from dropbox_sign.models.signature_request_response_signatures import (
    SignatureRequestResponseSignatures,
)
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union


class SignatureRequestResponse(BaseModel):
    """
    Contains information about a signature request.
    """  # noqa: E501

    test_mode: Optional[StrictBool] = Field(
        default=False,
        description="Whether this is a test signature request. Test requests have no legal value. Defaults to `false`.",
    )
    signature_request_id: Optional[StrictStr] = Field(
        default=None, description="The id of the SignatureRequest."
    )
    requester_email_address: Optional[StrictStr] = Field(
        default=None,
        description="The email address of the initiator of the SignatureRequest.",
    )
    title: Optional[StrictStr] = Field(
        default=None,
        description="The title the specified Account uses for the SignatureRequest.",
    )
    original_title: Optional[StrictStr] = Field(
        default=None, description="Default Label for account."
    )
    subject: Optional[StrictStr] = Field(
        default=None,
        description="The subject in the email that was initially sent to the signers.",
    )
    message: Optional[StrictStr] = Field(
        default=None,
        description="The custom message in the email that was initially sent to the signers.",
    )
    metadata: Optional[Dict[str, Any]] = Field(
        default=None, description="The metadata attached to the signature request."
    )
    created_at: Optional[StrictInt] = Field(
        default=None, description="Time the signature request was created."
    )
    expires_at: Optional[StrictInt] = Field(
        default=None,
        description="The time when the signature request will expire unsigned signatures. See [Signature Request Expiration Date](https://developers.hellosign.com/docs/signature-request/expiration/) for details.",
    )
    is_complete: Optional[StrictBool] = Field(
        default=None,
        description="Whether or not the SignatureRequest has been fully executed by all signers.",
    )
    is_declined: Optional[StrictBool] = Field(
        default=None,
        description="Whether or not the SignatureRequest has been declined by a signer.",
    )
    has_error: Optional[StrictBool] = Field(
        default=None,
        description="Whether or not an error occurred (either during the creation of the SignatureRequest or during one of the signings).",
    )
    files_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL where a copy of the request's documents can be downloaded.",
    )
    signing_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL where a signer, after authenticating, can sign the documents. This should only be used by users with existing Dropbox Sign accounts as they will be required to log in before signing.",
    )
    details_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL where the requester and the signers can view the current status of the SignatureRequest.",
    )
    cc_email_addresses: Optional[List[StrictStr]] = Field(
        default=None,
        description="A list of email addresses that were CCed on the SignatureRequest. They will receive a copy of the final PDF once all the signers have signed.",
    )
    signing_redirect_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL you want the signer redirected to after they successfully sign.",
    )
    final_copy_uri: Optional[StrictStr] = Field(
        default=None,
        description="The path where the completed document can be downloaded",
    )
    template_ids: Optional[List[StrictStr]] = Field(
        default=None,
        description="Templates IDs used in this SignatureRequest (if any).",
    )
    custom_fields: Optional[List[SignatureRequestResponseCustomFieldBase]] = Field(
        default=None,
        description="An array of Custom Field objects containing the name and type of each custom field.  * Text Field uses `SignatureRequestResponseCustomFieldText` * Checkbox Field uses `SignatureRequestResponseCustomFieldCheckbox`",
    )
    attachments: Optional[List[SignatureRequestResponseAttachment]] = Field(
        default=None, description="Signer attachments."
    )
    response_data: Optional[List[SignatureRequestResponseDataBase]] = Field(
        default=None,
        description="An array of form field objects containing the name, value, and type of each textbox or checkmark field filled in by the signers.",
    )
    signatures: Optional[List[SignatureRequestResponseSignatures]] = Field(
        default=None, description="An array of signature objects, 1 for each signer."
    )
    bulk_send_job_id: Optional[StrictStr] = Field(
        default=None,
        description="The ID of the Bulk Send job which sent the signature request, if applicable.",
    )
    __properties: ClassVar[List[str]] = [
        "test_mode",
        "signature_request_id",
        "requester_email_address",
        "title",
        "original_title",
        "subject",
        "message",
        "metadata",
        "created_at",
        "expires_at",
        "is_complete",
        "is_declined",
        "has_error",
        "files_url",
        "signing_url",
        "details_url",
        "cc_email_addresses",
        "signing_redirect_url",
        "final_copy_uri",
        "template_ids",
        "custom_fields",
        "attachments",
        "response_data",
        "signatures",
        "bulk_send_job_id",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SignatureRequestResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in custom_fields (list)
        _items = []
        if self.custom_fields:
            for _item_custom_fields in self.custom_fields:
                if _item_custom_fields:
                    _items.append(_item_custom_fields.to_dict())
            _dict["custom_fields"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in attachments (list)
        _items = []
        if self.attachments:
            for _item_attachments in self.attachments:
                if _item_attachments:
                    _items.append(_item_attachments.to_dict())
            _dict["attachments"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in response_data (list)
        _items = []
        if self.response_data:
            for _item_response_data in self.response_data:
                if _item_response_data:
                    _items.append(_item_response_data.to_dict())
            _dict["response_data"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in signatures (list)
        _items = []
        if self.signatures:
            for _item_signatures in self.signatures:
                if _item_signatures:
                    _items.append(_item_signatures.to_dict())
            _dict["signatures"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SignatureRequestResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "test_mode": (
                    obj.get("test_mode") if obj.get("test_mode") is not None else False
                ),
                "signature_request_id": obj.get("signature_request_id"),
                "requester_email_address": obj.get("requester_email_address"),
                "title": obj.get("title"),
                "original_title": obj.get("original_title"),
                "subject": obj.get("subject"),
                "message": obj.get("message"),
                "metadata": obj.get("metadata"),
                "created_at": obj.get("created_at"),
                "expires_at": obj.get("expires_at"),
                "is_complete": obj.get("is_complete"),
                "is_declined": obj.get("is_declined"),
                "has_error": obj.get("has_error"),
                "files_url": obj.get("files_url"),
                "signing_url": obj.get("signing_url"),
                "details_url": obj.get("details_url"),
                "cc_email_addresses": obj.get("cc_email_addresses"),
                "signing_redirect_url": obj.get("signing_redirect_url"),
                "final_copy_uri": obj.get("final_copy_uri"),
                "template_ids": obj.get("template_ids"),
                "custom_fields": (
                    [
                        SignatureRequestResponseCustomFieldBase.from_dict(_item)
                        for _item in obj["custom_fields"]
                    ]
                    if obj.get("custom_fields") is not None
                    else None
                ),
                "attachments": (
                    [
                        SignatureRequestResponseAttachment.from_dict(_item)
                        for _item in obj["attachments"]
                    ]
                    if obj.get("attachments") is not None
                    else None
                ),
                "response_data": (
                    [
                        SignatureRequestResponseDataBase.from_dict(_item)
                        for _item in obj["response_data"]
                    ]
                    if obj.get("response_data") is not None
                    else None
                ),
                "signatures": (
                    [
                        SignatureRequestResponseSignatures.from_dict(_item)
                        for _item in obj["signatures"]
                    ]
                    if obj.get("signatures") is not None
                    else None
                ),
                "bulk_send_job_id": obj.get("bulk_send_job_id"),
            }
        )
        return _obj

    @classmethod
    def init(cls, data: Any) -> Self:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        if isinstance(data, str):
            data = json.loads(data)

        return cls.from_dict(data)

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "test_mode": "(bool,)",
            "signature_request_id": "(str,)",
            "requester_email_address": "(str,)",
            "title": "(str,)",
            "original_title": "(str,)",
            "subject": "(str,)",
            "message": "(str,)",
            "metadata": "(object,)",
            "created_at": "(int,)",
            "expires_at": "(int,)",
            "is_complete": "(bool,)",
            "is_declined": "(bool,)",
            "has_error": "(bool,)",
            "files_url": "(str,)",
            "signing_url": "(str,)",
            "details_url": "(str,)",
            "cc_email_addresses": "(List[str],)",
            "signing_redirect_url": "(str,)",
            "final_copy_uri": "(str,)",
            "template_ids": "(List[str],)",
            "custom_fields": "(List[SignatureRequestResponseCustomFieldBase],)",
            "attachments": "(List[SignatureRequestResponseAttachment],)",
            "response_data": "(List[SignatureRequestResponseDataBase],)",
            "signatures": "(List[SignatureRequestResponseSignatures],)",
            "bulk_send_job_id": "(str,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in [
            "cc_email_addresses",
            "template_ids",
            "custom_fields",
            "attachments",
            "response_data",
            "signatures",
        ]



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from dropbox_sign.models.sub_form_fields_per_document_checkbox import (
        SubFormFieldsPerDocumentCheckbox,
    )
    from dropbox_sign.models.sub_form_fields_per_document_checkbox_merge import (
        SubFormFieldsPerDocumentCheckboxMerge,
    )
    from dropbox_sign.models.sub_form_fields_per_document_date_signed import (
        SubFormFieldsPerDocumentDateSigned,
    )
    from dropbox_sign.models.sub_form_fields_per_document_dropdown import (
        SubFormFieldsPerDocumentDropdown,
    )
    from dropbox_sign.models.sub_form_fields_per_document_hyperlink import (
        SubFormFieldsPerDocumentHyperlink,
    )
    from dropbox_sign.models.sub_form_fields_per_document_initials import (
        SubFormFieldsPerDocumentInitials,
    )
    from dropbox_sign.models.sub_form_fields_per_document_radio import (
        SubFormFieldsPerDocumentRadio,
    )
    from dropbox_sign.models.sub_form_fields_per_document_signature import (
        SubFormFieldsPerDocumentSignature,
    )
    from dropbox_sign.models.sub_form_fields_per_document_text import (
        SubFormFieldsPerDocumentText,
    )
    from dropbox_sign.models.sub_form_fields_per_document_text_merge import (
        SubFormFieldsPerDocumentTextMerge,
    )


class SubFormFieldsPerDocumentBase(BaseModel):
    """
    The fields that should appear on the document, expressed as an array of objects. (For more details you can read about it here: [Using Form Fields per Document](/docs/openapi/form-fields-per-document).)  **NOTE:** Fields like **text**, **dropdown**, **checkbox**, **radio**, and **hyperlink** have additional required and optional parameters. Check out the list of [additional parameters](/api/reference/constants/#form-fields-per-document) for these field types.  * Text Field use `SubFormFieldsPerDocumentText` * Dropdown Field use `SubFormFieldsPerDocumentDropdown` * Hyperlink Field use `SubFormFieldsPerDocumentHyperlink` * Checkbox Field use `SubFormFieldsPerDocumentCheckbox` * Radio Field use `SubFormFieldsPerDocumentRadio` * Signature Field use `SubFormFieldsPerDocumentSignature` * Date Signed Field use `SubFormFieldsPerDocumentDateSigned` * Initials Field use `SubFormFieldsPerDocumentInitials` * Text Merge Field use `SubFormFieldsPerDocumentTextMerge` * Checkbox Merge Field use `SubFormFieldsPerDocumentCheckboxMerge`
    """  # noqa: E501

    document_index: StrictInt = Field(
        description="Represents the integer index of the `file` or `file_url` document the field should be attached to."
    )
    api_id: StrictStr = Field(
        description="An identifier for the field that is unique across all documents in the request."
    )
    height: StrictInt = Field(description="Size of the field in pixels.")
    required: StrictBool = Field(description="Whether this field is required.")
    signer: Union[StrictStr, StrictInt] = Field(
        description="Signer index identified by the offset in the signers parameter (0-based indexing), indicating which signer should fill out the field.  **NOTE:** To set the value of the field as the preparer you must set this to `me_now`  **NOTE:** If type is `text-merge` or `checkbox-merge`, you must set this to sender in order to use pre-filled data."
    )
    type: StrictStr
    width: StrictInt = Field(description="Size of the field in pixels.")
    x: StrictInt = Field(description="Location coordinates of the field in pixels.")
    y: StrictInt = Field(description="Location coordinates of the field in pixels.")
    name: Optional[StrictStr] = Field(
        default=None, description="Display name for the field."
    )
    page: Optional[StrictInt] = Field(
        default=None,
        description="Page in the document where the field should be placed (requires documents be PDF files).  - When the page number parameter is supplied, the API will use the new coordinate system. - Check out the differences between both [coordinate systems](https://faq.hellosign.com/hc/en-us/articles/217115577) and how to use them.",
    )
    __properties: ClassVar[List[str]] = [
        "document_index",
        "api_id",
        "height",
        "required",
        "signer",
        "type",
        "width",
        "x",
        "y",
        "name",
        "page",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = "type"

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        "checkbox": "SubFormFieldsPerDocumentCheckbox",
        "checkbox-merge": "SubFormFieldsPerDocumentCheckboxMerge",
        "date_signed": "SubFormFieldsPerDocumentDateSigned",
        "dropdown": "SubFormFieldsPerDocumentDropdown",
        "hyperlink": "SubFormFieldsPerDocumentHyperlink",
        "initials": "SubFormFieldsPerDocumentInitials",
        "radio": "SubFormFieldsPerDocumentRadio",
        "signature": "SubFormFieldsPerDocumentSignature",
        "text": "SubFormFieldsPerDocumentText",
        "text-merge": "SubFormFieldsPerDocumentTextMerge",
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[
        Union[
            SubFormFieldsPerDocumentCheckbox,
            SubFormFieldsPerDocumentCheckboxMerge,
            SubFormFieldsPerDocumentDateSigned,
            SubFormFieldsPerDocumentDropdown,
            SubFormFieldsPerDocumentHyperlink,
            SubFormFieldsPerDocumentInitials,
            SubFormFieldsPerDocumentRadio,
            SubFormFieldsPerDocumentSignature,
            SubFormFieldsPerDocumentText,
            SubFormFieldsPerDocumentTextMerge,
        ]
    ]:
        """Create an instance of SubFormFieldsPerDocumentBase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[
        Union[
            SubFormFieldsPerDocumentCheckbox,
            SubFormFieldsPerDocumentCheckboxMerge,
            SubFormFieldsPerDocumentDateSigned,
            SubFormFieldsPerDocumentDropdown,
            SubFormFieldsPerDocumentHyperlink,
            SubFormFieldsPerDocumentInitials,
            SubFormFieldsPerDocumentRadio,
            SubFormFieldsPerDocumentSignature,
            SubFormFieldsPerDocumentText,
            SubFormFieldsPerDocumentTextMerge,
        ]
    ]:
        """Create an instance of SubFormFieldsPerDocumentBase from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type == "SubFormFieldsPerDocumentCheckbox":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_checkbox"
            ).SubFormFieldsPerDocumentCheckbox.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentCheckboxMerge":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_checkbox_merge"
            ).SubFormFieldsPerDocumentCheckboxMerge.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentDateSigned":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_date_signed"
            ).SubFormFieldsPerDocumentDateSigned.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentDropdown":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_dropdown"
            ).SubFormFieldsPerDocumentDropdown.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentHyperlink":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_hyperlink"
            ).SubFormFieldsPerDocumentHyperlink.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentInitials":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_initials"
            ).SubFormFieldsPerDocumentInitials.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentRadio":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_radio"
            ).SubFormFieldsPerDocumentRadio.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentSignature":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_signature"
            ).SubFormFieldsPerDocumentSignature.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentText":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_text"
            ).SubFormFieldsPerDocumentText.from_dict(obj)
        if object_type == "SubFormFieldsPerDocumentTextMerge":
            return import_module(
                "dropbox_sign.models.sub_form_fields_per_document_text_merge"
            ).SubFormFieldsPerDocumentTextMerge.from_dict(obj)

        raise ValueError(
            "SubFormFieldsPerDocumentBase failed to lookup discriminator value from "
            + json.dumps(obj)
            + ". Discriminator property name: "
            + cls.__discriminator_property_name
            + ", mapping: "
            + json.dumps(cls.__discriminator_value_class_map)
        )

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "document_index": "(int,)",
            "api_id": "(str,)",
            "height": "(int,)",
            "required": "(bool,)",
            "signer": "(int, str,)",
            "type": "(str,)",
            "width": "(int,)",
            "x": "(int,)",
            "y": "(int,)",
            "name": "(str,)",
            "page": "(int,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in []



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictBytes, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from dropbox_sign.models.sub_cc import SubCC
from dropbox_sign.models.sub_custom_field import SubCustomField
from dropbox_sign.models.sub_editor_options import SubEditorOptions
from dropbox_sign.models.sub_field_options import SubFieldOptions
from dropbox_sign.models.sub_signing_options import SubSigningOptions
from dropbox_sign.models.sub_unclaimed_draft_template_signer import (
    SubUnclaimedDraftTemplateSigner,
)
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union


class UnclaimedDraftCreateEmbeddedWithTemplateRequest(BaseModel):
    """
    UnclaimedDraftCreateEmbeddedWithTemplateRequest
    """  # noqa: E501

    client_id: StrictStr = Field(
        description="Client id of the app used to create the draft. Used to apply the branding and callback url defined for the app."
    )
    requester_email_address: StrictStr = Field(
        description="The email address of the user that should be designated as the requester of this draft."
    )
    template_ids: List[StrictStr] = Field(
        description="Use `template_ids` to create a SignatureRequest from one or more templates, in the order in which the templates will be used."
    )
    allow_decline: Optional[StrictBool] = Field(
        default=False,
        description="Allows signers to decline to sign a document if `true`. Defaults to `false`.",
    )
    allow_reassign: Optional[StrictBool] = Field(
        default=False,
        description="Allows signers to reassign their signature requests to other signers if set to `true`. Defaults to `false`.  **NOTE:** Only available for Premium plan and higher.",
    )
    ccs: Optional[List[SubCC]] = Field(
        default=None,
        description="Add CC email recipients. Required when a CC role exists for the Template.",
    )
    custom_fields: Optional[List[SubCustomField]] = Field(
        default=None,
        description="An array defining values and options for custom fields. Required when a custom field exists in the Template.",
    )
    editor_options: Optional[SubEditorOptions] = None
    field_options: Optional[SubFieldOptions] = None
    files: Optional[List[Union[StrictBytes, StrictStr, io.IOBase]]] = Field(
        default=None,
        description="Use `files[]` to append additional files to the signature request being created from the template. Dropbox Sign will parse the files for [text tags](https://app.hellosign.com/api/textTagsWalkthrough) and append it to the signature request. Text tags for signers not on the template(s) will be ignored.  **files** or **file_urls[]** is required, but not both.",
    )
    file_urls: Optional[List[StrictStr]] = Field(
        default=None,
        description="Use file_urls[] to append additional files to the signature request being created from the template. Dropbox Sign will download the file, then parse it for [text tags](https://app.hellosign.com/api/textTagsWalkthrough), and append to the signature request. Text tags for signers not on the template(s) will be ignored.  **files** or **file_urls[]** is required, but not both.",
    )
    force_signer_roles: Optional[StrictBool] = Field(
        default=False,
        description="Provide users the ability to review/edit the template signer roles.",
    )
    force_subject_message: Optional[StrictBool] = Field(
        default=False,
        description="Provide users the ability to review/edit the template subject and message.",
    )
    hold_request: Optional[StrictBool] = Field(
        default=False,
        description="The request from this draft will not automatically send to signers post-claim if set to 1. Requester must [release](/api/reference/operation/signatureRequestReleaseHold/) the request from hold when ready to send. Defaults to `false`.",
    )
    is_for_embedded_signing: Optional[StrictBool] = Field(
        default=False,
        description="The request created from this draft will also be signable in embedded mode if set to `true`. Defaults to `false`.",
    )
    message: Optional[Annotated[str, Field(strict=True, max_length=5000)]] = Field(
        default=None,
        description="The custom message in the email that will be sent to the signers.",
    )
    metadata: Optional[Dict[str, Any]] = Field(
        default=None,
        description="Key-value data that should be attached to the signature request. This metadata is included in all API responses and events involving the signature request. For example, use the metadata field to store a signer's order number for look up when receiving events for the signature request.  Each request can include up to 10 metadata keys (or 50 nested metadata keys), with key names up to 40 characters long and values up to 1000 characters long.",
    )
    preview_only: Optional[StrictBool] = Field(
        default=False,
        description="This allows the requester to enable the preview experience (i.e. does not allow the requester's end user to add any additional fields via the editor).  - `preview_only=true`: Allows requesters to enable the preview only experience. - `preview_only=false`: Allows requesters to disable the preview only experience.  **NOTE:** This parameter overwrites `show_preview=1` (if set).",
    )
    requesting_redirect_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL you want signers redirected to after they successfully request a signature.",
    )
    show_preview: Optional[StrictBool] = Field(
        default=False,
        description="This allows the requester to enable the editor/preview experience.  - `show_preview=true`: Allows requesters to enable the editor/preview experience. - `show_preview=false`: Allows requesters to disable the editor/preview experience.",
    )
    show_progress_stepper: Optional[StrictBool] = Field(
        default=True,
        description="When only one step remains in the signature request process and this parameter is set to `false` then the progress stepper will be hidden.",
    )
    signers: Optional[List[SubUnclaimedDraftTemplateSigner]] = Field(
        default=None,
        description="Add Signers to your Templated-based Signature Request.",
    )
    signing_options: Optional[SubSigningOptions] = None
    signing_redirect_url: Optional[StrictStr] = Field(
        default=None,
        description="The URL you want signers redirected to after they successfully sign.",
    )
    skip_me_now: Optional[StrictBool] = Field(
        default=False,
        description='Disables the "Me (Now)" option for the person preparing the document. Does not work with type `send_document`. Defaults to `false`.',
    )
    subject: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(
        default=None,
        description="The subject in the email that will be sent to the signers.",
    )
    test_mode: Optional[StrictBool] = Field(
        default=False,
        description="Whether this is a test, the signature request created from this draft will not be legally binding if set to `true`. Defaults to `false`.",
    )
    title: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(
        default=None,
        description="The title you want to assign to the SignatureRequest.",
    )
    populate_auto_fill_fields: Optional[StrictBool] = Field(
        default=False,
        description="Controls whether [auto fill fields](https://faq.hellosign.com/hc/en-us/articles/360051467511-Auto-Fill-Fields) can automatically populate a signer's information during signing.  **NOTE:** Keep your signer's information safe by ensuring that the _signer on your signature request is the intended party_ before using this feature.",
    )
    allow_ccs: Optional[StrictBool] = Field(
        default=False,
        description="This allows the requester to specify whether the user is allowed to provide email addresses to CC when claiming the draft.",
    )
    __properties: ClassVar[List[str]] = [
        "client_id",
        "requester_email_address",
        "template_ids",
        "allow_decline",
        "allow_reassign",
        "ccs",
        "custom_fields",
        "editor_options",
        "field_options",
        "files",
        "file_urls",
        "force_signer_roles",
        "force_subject_message",
        "hold_request",
        "is_for_embedded_signing",
        "message",
        "metadata",
        "preview_only",
        "requesting_redirect_url",
        "show_preview",
        "show_progress_stepper",
        "signers",
        "signing_options",
        "signing_redirect_url",
        "skip_me_now",
        "subject",
        "test_mode",
        "title",
        "populate_auto_fill_fields",
        "allow_ccs",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UnclaimedDraftCreateEmbeddedWithTemplateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in ccs (list)
        _items = []
        if self.ccs:
            for _item_ccs in self.ccs:
                if _item_ccs:
                    _items.append(_item_ccs.to_dict())
            _dict["ccs"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in custom_fields (list)
        _items = []
        if self.custom_fields:
            for _item_custom_fields in self.custom_fields:
                if _item_custom_fields:
                    _items.append(_item_custom_fields.to_dict())
            _dict["custom_fields"] = _items
        # override the default output from pydantic by calling `to_dict()` of editor_options
        if self.editor_options:
            _dict["editor_options"] = self.editor_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of field_options
        if self.field_options:
            _dict["field_options"] = self.field_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in signers (list)
        _items = []
        if self.signers:
            for _item_signers in self.signers:
                if _item_signers:
                    _items.append(_item_signers.to_dict())
            _dict["signers"] = _items
        # override the default output from pydantic by calling `to_dict()` of signing_options
        if self.signing_options:
            _dict["signing_options"] = self.signing_options.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UnclaimedDraftCreateEmbeddedWithTemplateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "client_id": obj.get("client_id"),
                "requester_email_address": obj.get("requester_email_address"),
                "template_ids": obj.get("template_ids"),
                "allow_decline": (
                    obj.get("allow_decline")
                    if obj.get("allow_decline") is not None
                    else False
                ),
                "allow_reassign": (
                    obj.get("allow_reassign")
                    if obj.get("allow_reassign") is not None
                    else False
                ),
                "ccs": (
                    [SubCC.from_dict(_item) for _item in obj["ccs"]]
                    if obj.get("ccs") is not None
                    else None
                ),
                "custom_fields": (
                    [SubCustomField.from_dict(_item) for _item in obj["custom_fields"]]
                    if obj.get("custom_fields") is not None
                    else None
                ),
                "editor_options": (
                    SubEditorOptions.from_dict(obj["editor_options"])
                    if obj.get("editor_options") is not None
                    else None
                ),
                "field_options": (
                    SubFieldOptions.from_dict(obj["field_options"])
                    if obj.get("field_options") is not None
                    else None
                ),
                "files": obj.get("files"),
                "file_urls": obj.get("file_urls"),
                "force_signer_roles": (
                    obj.get("force_signer_roles")
                    if obj.get("force_signer_roles") is not None
                    else False
                ),
                "force_subject_message": (
                    obj.get("force_subject_message")
                    if obj.get("force_subject_message") is not None
                    else False
                ),
                "hold_request": (
                    obj.get("hold_request")
                    if obj.get("hold_request") is not None
                    else False
                ),
                "is_for_embedded_signing": (
                    obj.get("is_for_embedded_signing")
                    if obj.get("is_for_embedded_signing") is not None
                    else False
                ),
                "message": obj.get("message"),
                "metadata": obj.get("metadata"),
                "preview_only": (
                    obj.get("preview_only")
                    if obj.get("preview_only") is not None
                    else False
                ),
                "requesting_redirect_url": obj.get("requesting_redirect_url"),
                "show_preview": (
                    obj.get("show_preview")
                    if obj.get("show_preview") is not None
                    else False
                ),
                "show_progress_stepper": (
                    obj.get("show_progress_stepper")
                    if obj.get("show_progress_stepper") is not None
                    else True
                ),
                "signers": (
                    [
                        SubUnclaimedDraftTemplateSigner.from_dict(_item)
                        for _item in obj["signers"]
                    ]
                    if obj.get("signers") is not None
                    else None
                ),
                "signing_options": (
                    SubSigningOptions.from_dict(obj["signing_options"])
                    if obj.get("signing_options") is not None
                    else None
                ),
                "signing_redirect_url": obj.get("signing_redirect_url"),
                "skip_me_now": (
                    obj.get("skip_me_now")
                    if obj.get("skip_me_now") is not None
                    else False
                ),
                "subject": obj.get("subject"),
                "test_mode": (
                    obj.get("test_mode") if obj.get("test_mode") is not None else False
                ),
                "title": obj.get("title"),
                "populate_auto_fill_fields": (
                    obj.get("populate_auto_fill_fields")
                    if obj.get("populate_auto_fill_fields") is not None
                    else False
                ),
                "allow_ccs": (
                    obj.get("allow_ccs") if obj.get("allow_ccs") is not None else False
                ),
            }
        )
        return _obj

    @classmethod
    def init(cls, data: Any) -> Self:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        if isinstance(data, str):
            data = json.loads(data)

        return cls.from_dict(data)

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "client_id": "(str,)",
            "requester_email_address": "(str,)",
            "template_ids": "(List[str],)",
            "allow_decline": "(bool,)",
            "allow_reassign": "(bool,)",
            "ccs": "(List[SubCC],)",
            "custom_fields": "(List[SubCustomField],)",
            "editor_options": "(SubEditorOptions,)",
            "field_options": "(SubFieldOptions,)",
            "files": "(List[io.IOBase],)",
            "file_urls": "(List[str],)",
            "force_signer_roles": "(bool,)",
            "force_subject_message": "(bool,)",
            "hold_request": "(bool,)",
            "is_for_embedded_signing": "(bool,)",
            "message": "(str,)",
            "metadata": "(Dict[str, object],)",
            "preview_only": "(bool,)",
            "requesting_redirect_url": "(str,)",
            "show_preview": "(bool,)",
            "show_progress_stepper": "(bool,)",
            "signers": "(List[SubUnclaimedDraftTemplateSigner],)",
            "signing_options": "(SubSigningOptions,)",
            "signing_redirect_url": "(str,)",
            "skip_me_now": "(bool,)",
            "subject": "(str,)",
            "test_mode": "(bool,)",
            "title": "(str,)",
            "populate_auto_fill_fields": "(bool,)",
            "allow_ccs": "(bool,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in [
            "template_ids",
            "ccs",
            "custom_fields",
            "files",
            "file_urls",
            "signers",
        ]



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from setuptools import setup, find_packages  # noqa: H301
from pathlib import Path

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools
NAME = "dropbox-sign"
VERSION = "1.7.0"
PYTHON_REQUIRES = ">=3.7"
REQUIRES = [
    "urllib3 >= 1.25.3, < 2.1.0",
    "python-dateutil",
    "pydantic >= 2",
    "typing-extensions >= 4.7.1",
]

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name=NAME,
    version=VERSION,
    description="Dropbox Sign API",
    author="Dropbox Sign API Team",
    author_email="apisupport@hellosign.com",
    url="https://github.com/hellosign/dropbox-sign-python",
    keywords=["OpenAPI", "OpenAPI-Generator", "Dropbox Sign API"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    license="MIT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    package_data={"dropbox_sign": ["py.typed"]},
)



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from dropbox_sign.models.signature_request_response_custom_field_checkbox import (
        SignatureRequestResponseCustomFieldCheckbox,
    )
    from dropbox_sign.models.signature_request_response_custom_field_text import (
        SignatureRequestResponseCustomFieldText,
    )


class SignatureRequestResponseCustomFieldBase(BaseModel):
    """
    An array of Custom Field objects containing the name and type of each custom field.  * Text Field uses `SignatureRequestResponseCustomFieldText` * Checkbox Field uses `SignatureRequestResponseCustomFieldCheckbox`
    """  # noqa: E501

    type: StrictStr = Field(
        description="The type of this Custom Field. Only 'text' and 'checkbox' are currently supported."
    )
    name: StrictStr = Field(description="The name of the Custom Field.")
    required: Optional[StrictBool] = Field(
        default=None, description="A boolean value denoting if this field is required."
    )
    api_id: Optional[StrictStr] = Field(
        default=None, description="The unique ID for this field."
    )
    editor: Optional[StrictStr] = Field(
        default=None,
        description="The name of the Role that is able to edit this field.",
    )
    __properties: ClassVar[List[str]] = ["type", "name", "required", "api_id", "editor"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = "type"

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        "checkbox": "SignatureRequestResponseCustomFieldCheckbox",
        "text": "SignatureRequestResponseCustomFieldText",
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[
        Union[
            SignatureRequestResponseCustomFieldCheckbox,
            SignatureRequestResponseCustomFieldText,
        ]
    ]:
        """Create an instance of SignatureRequestResponseCustomFieldBase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[
        Union[
            SignatureRequestResponseCustomFieldCheckbox,
            SignatureRequestResponseCustomFieldText,
        ]
    ]:
        """Create an instance of SignatureRequestResponseCustomFieldBase from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type == "SignatureRequestResponseCustomFieldCheckbox":
            return import_module(
                "dropbox_sign.models.signature_request_response_custom_field_checkbox"
            ).SignatureRequestResponseCustomFieldCheckbox.from_dict(obj)
        if object_type == "SignatureRequestResponseCustomFieldText":
            return import_module(
                "dropbox_sign.models.signature_request_response_custom_field_text"
            ).SignatureRequestResponseCustomFieldText.from_dict(obj)

        raise ValueError(
            "SignatureRequestResponseCustomFieldBase failed to lookup discriminator value from "
            + json.dumps(obj)
            + ". Discriminator property name: "
            + cls.__discriminator_property_name
            + ", mapping: "
            + json.dumps(cls.__discriminator_value_class_map)
        )

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "type": "(str,)",
            "name": "(str,)",
            "required": "(bool,)",
            "api_id": "(str,)",
            "editor": "(str,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in []



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union


class SignatureRequestResponseSignatures(BaseModel):
    """
    An array of signature objects, 1 for each signer.
    """  # noqa: E501

    signature_id: Optional[StrictStr] = Field(
        default=None, description="Signature identifier."
    )
    signer_group_guid: Optional[StrictStr] = Field(
        default=None, description="Signer Group GUID"
    )
    signer_email_address: Optional[StrictStr] = Field(
        default=None, description="The email address of the signer."
    )
    signer_name: Optional[StrictStr] = Field(
        default=None, description="The name of the signer."
    )
    signer_role: Optional[StrictStr] = Field(
        default=None, description="The role of the signer."
    )
    order: Optional[StrictInt] = Field(
        default=None,
        description="If signer order is assigned this is the 0-based index for this signer.",
    )
    status_code: Optional[StrictStr] = Field(
        default=None,
        description="The current status of the signature. eg: awaiting_signature, signed, declined.",
    )
    decline_reason: Optional[StrictStr] = Field(
        default=None,
        description="The reason provided by the signer for declining the request.",
    )
    signed_at: Optional[StrictInt] = Field(
        default=None, description="Time that the document was signed or null."
    )
    last_viewed_at: Optional[StrictInt] = Field(
        default=None,
        description="The time that the document was last viewed by this signer or null.",
    )
    last_reminded_at: Optional[StrictInt] = Field(
        default=None,
        description="The time the last reminder email was sent to the signer or null.",
    )
    has_pin: Optional[StrictBool] = Field(
        default=None,
        description="Boolean to indicate whether this signature requires a PIN to access.",
    )
    has_sms_auth: Optional[StrictBool] = Field(
        default=None,
        description="Boolean to indicate whether this signature has SMS authentication enabled.",
    )
    has_sms_delivery: Optional[StrictBool] = Field(
        default=None,
        description="Boolean to indicate whether this signature has SMS delivery enabled.",
    )
    sms_phone_number: Optional[StrictStr] = Field(
        default=None,
        description="The SMS phone number used for authentication or signature request delivery.",
    )
    reassigned_by: Optional[StrictStr] = Field(
        default=None,
        description="Email address of original signer who reassigned to this signer.",
    )
    reassignment_reason: Optional[StrictStr] = Field(
        default=None,
        description="Reason provided by original signer who reassigned to this signer.",
    )
    reassigned_from: Optional[StrictStr] = Field(
        default=None, description="Previous signature identifier."
    )
    error: Optional[StrictStr] = Field(
        default=None, description="Error message pertaining to this signer, or null."
    )
    __properties: ClassVar[List[str]] = [
        "signature_id",
        "signer_group_guid",
        "signer_email_address",
        "signer_name",
        "signer_role",
        "order",
        "status_code",
        "decline_reason",
        "signed_at",
        "last_viewed_at",
        "last_reminded_at",
        "has_pin",
        "has_sms_auth",
        "has_sms_delivery",
        "sms_phone_number",
        "reassigned_by",
        "reassignment_reason",
        "reassigned_from",
        "error",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SignatureRequestResponseSignatures from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SignatureRequestResponseSignatures from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "signature_id": obj.get("signature_id"),
                "signer_group_guid": obj.get("signer_group_guid"),
                "signer_email_address": obj.get("signer_email_address"),
                "signer_name": obj.get("signer_name"),
                "signer_role": obj.get("signer_role"),
                "order": obj.get("order"),
                "status_code": obj.get("status_code"),
                "decline_reason": obj.get("decline_reason"),
                "signed_at": obj.get("signed_at"),
                "last_viewed_at": obj.get("last_viewed_at"),
                "last_reminded_at": obj.get("last_reminded_at"),
                "has_pin": obj.get("has_pin"),
                "has_sms_auth": obj.get("has_sms_auth"),
                "has_sms_delivery": obj.get("has_sms_delivery"),
                "sms_phone_number": obj.get("sms_phone_number"),
                "reassigned_by": obj.get("reassigned_by"),
                "reassignment_reason": obj.get("reassignment_reason"),
                "reassigned_from": obj.get("reassigned_from"),
                "error": obj.get("error"),
            }
        )
        return _obj

    @classmethod
    def init(cls, data: Any) -> Self:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        if isinstance(data, str):
            data = json.loads(data)

        return cls.from_dict(data)

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "signature_id": "(str,)",
            "signer_group_guid": "(str,)",
            "signer_email_address": "(str,)",
            "signer_name": "(str,)",
            "signer_role": "(str,)",
            "order": "(int,)",
            "status_code": "(str,)",
            "decline_reason": "(str,)",
            "signed_at": "(int,)",
            "last_viewed_at": "(int,)",
            "last_reminded_at": "(int,)",
            "has_pin": "(bool,)",
            "has_sms_auth": "(bool,)",
            "has_sms_delivery": "(bool,)",
            "sms_phone_number": "(str,)",
            "reassigned_by": "(str,)",
            "reassignment_reason": "(str,)",
            "reassigned_from": "(str,)",
            "error": "(str,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in []



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import (
    ConfigDict,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing import Any, ClassVar, Dict, List, Optional
from dropbox_sign.models.sub_form_fields_per_document_base import (
    SubFormFieldsPerDocumentBase,
)
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union


class SubFormFieldsPerDocumentText(SubFormFieldsPerDocumentBase):
    """
    This class extends `SubFormFieldsPerDocumentBase`.
    """  # noqa: E501

    type: StrictStr = Field(
        description="A text input field. Use the `SubFormFieldsPerDocumentText` class."
    )
    placeholder: Optional[StrictStr] = Field(
        default=None, description="Placeholder value for text field."
    )
    auto_fill_type: Optional[StrictStr] = Field(
        default=None,
        description="Auto fill type for populating fields automatically. Check out the list of [auto fill types](/api/reference/constants/#auto-fill-types) to learn more about the possible values.",
    )
    link_id: Optional[StrictStr] = Field(
        default=None,
        description="Link two or more text fields. Enter data into one linked text field, which automatically fill all other linked text fields.",
    )
    masked: Optional[StrictBool] = Field(
        default=None,
        description="Masks entered data. For more information see [Masking sensitive information](https://faq.hellosign.com/hc/en-us/articles/360040742811-Masking-sensitive-information). `true` for masking the data in a text field, otherwise `false`.",
    )
    validation_type: Optional[StrictStr] = Field(
        default=None,
        description="Each text field may contain a `validation_type` parameter. Check out the list of [validation types](https://faq.hellosign.com/hc/en-us/articles/217115577) to learn more about the possible values.  **NOTE:** When using `custom_regex` you are required to pass a second parameter `validation_custom_regex` and you can optionally provide `validation_custom_regex_format_label` for the error message the user will see in case of an invalid value.",
    )
    validation_custom_regex: Optional[StrictStr] = None
    validation_custom_regex_format_label: Optional[StrictStr] = None
    content: Optional[StrictStr] = Field(
        default=None, description="Content of a `me_now` text field"
    )
    font_family: Optional[StrictStr] = Field(
        default=None, description="Font family for the field."
    )
    font_size: Optional[StrictInt] = Field(
        default=12,
        description="The initial px font size for the field contents. Can be any integer value between `7` and `49`.  **NOTE:** Font size may be reduced during processing in order to fit the contents within the dimensions of the field.",
    )
    __properties: ClassVar[List[str]] = [
        "document_index",
        "api_id",
        "height",
        "required",
        "signer",
        "type",
        "width",
        "x",
        "y",
        "name",
        "page",
        "placeholder",
        "auto_fill_type",
        "link_id",
        "masked",
        "validation_type",
        "validation_custom_regex",
        "validation_custom_regex_format_label",
        "content",
        "font_family",
        "font_size",
    ]

    @field_validator("validation_type")
    def validation_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(
            [
                "numbers_only",
                "letters_only",
                "phone_number",
                "bank_routing_number",
                "bank_account_number",
                "email_address",
                "zip_code",
                "social_security_number",
                "employer_identification_number",
                "custom_regex",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('numbers_only', 'letters_only', 'phone_number', 'bank_routing_number', 'bank_account_number', 'email_address', 'zip_code', 'social_security_number', 'employer_identification_number', 'custom_regex')"
            )
        return value

    @field_validator("font_family")
    def font_family_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(
            [
                "helvetica",
                "arial",
                "courier",
                "calibri",
                "cambria",
                "georgia",
                "times",
                "trebuchet",
                "verdana",
                "roboto",
                "robotoMono",
                "notoSans",
                "notoSerif",
                "notoCJK-JP-Regular",
                "notoHebrew-Regular",
                "notoSanThaiMerged",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('helvetica', 'arial', 'courier', 'calibri', 'cambria', 'georgia', 'times', 'trebuchet', 'verdana', 'roboto', 'robotoMono', 'notoSans', 'notoSerif', 'notoCJK-JP-Regular', 'notoHebrew-Regular', 'notoSanThaiMerged')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubFormFieldsPerDocumentText from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubFormFieldsPerDocumentText from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "document_index": obj.get("document_index"),
                "api_id": obj.get("api_id"),
                "height": obj.get("height"),
                "required": obj.get("required"),
                "signer": obj.get("signer"),
                "type": obj.get("type") if obj.get("type") is not None else "text",
                "width": obj.get("width"),
                "x": obj.get("x"),
                "y": obj.get("y"),
                "name": obj.get("name"),
                "page": obj.get("page"),
                "placeholder": obj.get("placeholder"),
                "auto_fill_type": obj.get("auto_fill_type"),
                "link_id": obj.get("link_id"),
                "masked": obj.get("masked"),
                "validation_type": obj.get("validation_type"),
                "validation_custom_regex": obj.get("validation_custom_regex"),
                "validation_custom_regex_format_label": obj.get(
                    "validation_custom_regex_format_label"
                ),
                "content": obj.get("content"),
                "font_family": obj.get("font_family"),
                "font_size": (
                    obj.get("font_size") if obj.get("font_size") is not None else 12
                ),
            }
        )
        return _obj

    @classmethod
    def init(cls, data: Any) -> Self:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        if isinstance(data, str):
            data = json.loads(data)

        return cls.from_dict(data)

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "type": "(str,)",
            "document_index": "(int,)",
            "api_id": "(str,)",
            "height": "(int,)",
            "required": "(bool,)",
            "signer": "(int, str,)",
            "width": "(int,)",
            "x": "(int,)",
            "y": "(int,)",
            "placeholder": "(str,)",
            "auto_fill_type": "(str,)",
            "link_id": "(str,)",
            "masked": "(bool,)",
            "validation_type": "(str,)",
            "validation_custom_regex": "(str,)",
            "validation_custom_regex_format_label": "(str,)",
            "content": "(str,)",
            "font_family": "(str,)",
            "font_size": "(int,)",
            "name": "(str,)",
            "page": "(int,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in []



import json
import unittest

from dropbox_sign import ApiClient, Configuration, models
from test_utils import get_fixture_data


class TestFixtures(unittest.TestCase):
    def test_is_valid(self):
        configuration = Configuration()
        api_client = ApiClient(configuration)

        fixtures = [
            "AccountCreateRequest",
            "AccountUpdateRequest",
            "AccountVerifyRequest",
            "ApiAppCreateRequest",
            "ApiAppUpdateRequest",
            "EmbeddedEditUrlRequest",
            "OAuthTokenGenerateRequest",
            "OAuthTokenRefreshRequest",
            "ReportCreateRequest",
            "SignatureRequestBulkCreateEmbeddedWithTemplateRequest",
            "SignatureRequestBulkSendWithTemplateRequest",
            "SignatureRequestCreateEmbeddedRequest",
            "SignatureRequestCreateEmbeddedWithTemplateRequest",
            "SignatureRequestRemindRequest",
            "SignatureRequestSendRequest",
            "SignatureRequestSendWithTemplateRequest",
            "SignatureRequestUpdateRequest",
            "TeamAddMemberRequest",
            "TeamCreateRequest",
            "TeamRemoveMemberRequest",
            "TeamUpdateRequest",
            "TemplateAddUserRequest",
            "TemplateCreateEmbeddedDraftRequest",
            "TemplateRemoveUserRequest",
            "TemplateUpdateFilesRequest",
            "UnclaimedDraftCreateEmbeddedRequest",
            "UnclaimedDraftCreateEmbeddedWithTemplateRequest",
            "UnclaimedDraftCreateRequest",
            "UnclaimedDraftEditAndResendRequest",
        ]

        for fixture in fixtures:
            fixture_data = get_fixture_data(fixture)

            for key, fixt_data in fixture_data.items():
                class_type = eval(f"models.{fixture}")
                yanked_files = {}
                data = {}

                for param, param_value in class_type.openapi_types().items():
                    if param not in fixt_data.keys():
                        continue

                    data[param] = fixt_data[param]

                obj = api_client.deserialize(
                    response_text=json.dumps(data),
                    response_type=class_type,
                    content_type="application/json",
                )

                for yanked_key, yanked_file in yanked_files.items():
                    obj[yanked_key] = yanked_file

                self.assertEqual(obj.__class__.__name__, class_type.__name__)


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    Dropbox Sign API

    Dropbox Sign v3 API

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set, Tuple
from typing_extensions import Self
import io
from pydantic import StrictBool
from typing import Union


class SubCustomField(BaseModel):
    """
    When used together with merge fields, `custom_fields` allows users to add pre-filled data to their signature requests.  Pre-filled data can be used with \"send-once\" signature requests by adding merge fields with `form_fields_per_document` or [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) while passing values back with `custom_fields` together in one API call.  For using pre-filled on repeatable signature requests, merge fields are added to templates in the Dropbox Sign UI or by calling [/template/create_embedded_draft](/api/reference/operation/templateCreateEmbeddedDraft) and then passing `custom_fields` on subsequent signature requests referencing that template.
    """  # noqa: E501

    name: StrictStr = Field(
        description="The name of a custom field. When working with pre-filled data, the custom field's name must have a matching merge field name or the field will remain empty on the document during signing."
    )
    editor: Optional[StrictStr] = Field(
        default=None,
        description="Used to create editable merge fields. When the value matches a role passed in with `signers`, that role can edit the data that was pre-filled to that field. This field is optional, but required when this custom field object is set to `required = true`.  **NOTE:** Editable merge fields are only supported for single signer requests (or the first signer in ordered signature requests). If used when there are multiple signers in an unordered signature request, the editor value is ignored and the field won't be editable.",
    )
    required: Optional[StrictBool] = Field(
        default=False,
        description="Used to set an editable merge field when working with pre-filled data. When `true`, the custom field must specify a signer role in `editor`.",
    )
    value: Optional[StrictStr] = Field(
        default=None,
        description='The string that resolves (aka "pre-fills") to the merge field on the final document(s) used for signing.',
    )
    __properties: ClassVar[List[str]] = ["name", "editor", "required", "value"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    def to_json_form_params(
        self, excluded_fields: Set[str] = None
    ) -> List[Tuple[str, str]]:
        data: List[Tuple[str, str]] = []

        for key, value in self.to_dict(excluded_fields).items():
            if isinstance(value, (int, str, bool)):
                data.append((key, value))
            else:
                data.append((key, json.dumps(value, ensure_ascii=False)))

        return data

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubCustomField from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self, excluded_fields: Set[str] = None) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubCustomField from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "editor": obj.get("editor"),
                "required": (
                    obj.get("required") if obj.get("required") is not None else False
                ),
                "value": obj.get("value"),
            }
        )
        return _obj

    @classmethod
    def init(cls, data: Any) -> Self:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        if isinstance(data, str):
            data = json.loads(data)

        return cls.from_dict(data)

    @classmethod
    def openapi_types(cls) -> Dict[str, str]:
        return {
            "name": "(str,)",
            "editor": "(str,)",
            "required": "(bool,)",
            "value": "(str,)",
        }

    @classmethod
    def openapi_type_is_array(cls, property_name: str) -> bool:
        return property_name in []
